@@ -28,11 +28,10 @@ fi
 
 cmake \
   -DCMAKE_CXX_COMPILER="${CXX}" \
-  -DCMAKE_CXX_FLAGS="${CXXFLAGS} -Werror" \
+  -DPOLYBAR_FLAGS="${CXXFLAGS} -Werror" \
   -DCMAKE_BUILD_TYPE="${BUILD_TYPE}" \
   -DBUILD_TESTS:BOOL="${BUILD_TESTS:-OFF}" \
-  -DBUILD_DOC:BOOL="${BUILD_DOC:-OFF}" \
-  -DWITH_XRANDR=ON \
+  -DBUILD_DOC:BOOL=OFF \
   -DENABLE_PULSEAUDIO="${ENABLE_PULSEAUDIO:-OFF}" \
   -DENABLE_NETWORK="${ENABLE_NETWORK:-OFF}" \
   -DENABLE_MPD="${ENABLE_MPD:-OFF}" \
