@@ -33,8 +33,10 @@ class config {
   const string& filepath() const;
   string section() const;
 
+  static constexpr const char* BAR_PREFIX = "bar/";
+
   /**
-   * \brief Instruct the config to connect to the xresource manager
+   * @brief Instruct the config to connect to the xresource manager
    */
   void use_xrm();
 
@@ -105,12 +107,38 @@ class config {
       return dereference<T>(move(section), move(key), move(string_value), move(result));
     } catch (const key_error& err) {
       return default_value;
-    } catch (const value_error& err) {
+    } catch (const std::exception& err) {
       m_log.err("Invalid value for \"%s.%s\", using default value (reason: %s)", section, key, err.what());
       return default_value;
     }
   }
 
+  /**
+   * Get list of key-value pairs starting with a prefix by section.
+   *
+   * Eg: if you have in config `env-FOO = bar`,
+   *    get_with_prefix(section, "env-") will return [{"FOO", "bar"}]
+   */
+  template <typename T = string>
+  vector<pair<string, T>> get_with_prefix(const string& section, const string& key_prefix) const {
+    auto it = m_sections.find(section);
+    if (it == m_sections.end()) {
+      throw key_error("Missing section \"" + section + "\"");
+    }
+
+    vector<pair<string, T>> list;
+    for (const auto& kv_pair : it->second) {
+      const auto& key = kv_pair.first;
+
+      if (key.substr(0, key_prefix.size()) == key_prefix) {
+        const T& val = get<T>(section, key);
+        list.emplace_back(key.substr(key_prefix.size()), val);
+      }
+    }
+
+    return list;
+  }
+
   /**
    * Get list of values for the current bar by name
    */
@@ -168,7 +196,7 @@ class config {
         }
       } catch (const key_error& err) {
         break;
-      } catch (const value_error& err) {
+      } catch (const std::exception& err) {
         m_log.err("Invalid value in list \"%s.%s\", using list as-is (reason: %s)", section, key, err.what());
         return default_value;
       }
@@ -182,6 +210,8 @@ class config {
     return default_value;
   }
 
+  void ignore_key(const string& section, const string& key) const;
+
   /**
    * Attempt to load value using the deprecated key name. If successful show a
    * warning message. If it fails load the value using the new key and given
@@ -199,7 +229,7 @@ class config {
   }
 
   /**
-   * \see deprecated<T>
+   * @see deprecated<T>
    */
   template <typename T = string>
   T deprecated_list(const string& section, const string& old, const string& newkey, const vector<T>& fallback) const {
