@@ -18,15 +18,15 @@ namespace drawtypes {
     bool zpad{false};
   };
 
-  class label : public non_copyable_mixin<label> {
+  class label : public non_copyable_mixin {
    public:
     rgba m_foreground{};
     rgba m_background{};
     rgba m_underline{};
     rgba m_overline{};
     int m_font{0};
-    side_values m_padding{0U, 0U};
-    side_values m_margin{0U, 0U};
+    side_values m_padding{ZERO_SPACE, ZERO_SPACE};
+    side_values m_margin{ZERO_SPACE, ZERO_SPACE};
 
     size_t m_minlen{0};
     /*
@@ -40,15 +40,15 @@ namespace drawtypes {
     alignment m_alignment{alignment::LEFT};
     bool m_ellipsis{true};
 
-    explicit label(string text, int font) : m_font(font), m_text(text), m_tokenized(m_text) {}
+    explicit label(string text, int font) : m_font(font), m_text(move(text)), m_tokenized(m_text) {}
     explicit label(string text, rgba foreground = rgba{}, rgba background = rgba{}, rgba underline = rgba{},
-        rgba overline = rgba{}, int font = 0, struct side_values padding = {0U, 0U},
-        struct side_values margin = {0U, 0U}, int minlen = 0, size_t maxlen = 0_z,
+        rgba overline = rgba{}, int font = 0, side_values padding = {ZERO_SPACE, ZERO_SPACE},
+        side_values margin = {ZERO_SPACE, ZERO_SPACE}, int minlen = 0, size_t maxlen = 0_z,
         alignment label_alignment = alignment::LEFT, bool ellipsis = true, vector<token>&& tokens = {})
-        : m_foreground(foreground)
-        , m_background(background)
-        , m_underline(underline)
-        , m_overline(overline)
+        : m_foreground(move(foreground))
+        , m_background(move(background))
+        , m_underline(move(underline))
+        , m_overline(move(overline))
         , m_font(font)
         , m_padding(padding)
         , m_margin(margin)
@@ -56,14 +56,14 @@ namespace drawtypes {
         , m_maxlen(maxlen)
         , m_alignment(label_alignment)
         , m_ellipsis(ellipsis)
-        , m_text(text)
+        , m_text(move(text))
         , m_tokenized(m_text)
         , m_tokens(forward<vector<token>>(tokens)) {
       assert(!m_ellipsis || (m_maxlen == 0 || m_maxlen >= 3));
     }
 
     string get() const;
-    operator bool();
+    explicit operator bool();
     label_t clone();
     void clear();
     void reset_tokens();
@@ -81,6 +81,6 @@ namespace drawtypes {
 
   label_t load_label(const config& conf, const string& section, string name, bool required = true, string def = ""s);
   label_t load_optional_label(const config& conf, string section, string name, string def = ""s);
-}  // namespace drawtypes
+} // namespace drawtypes
 
 POLYBAR_NS_END
