@@ -8,15 +8,17 @@
 POLYBAR_NS
 
 namespace drawtypes {
-  class ramp : public non_copyable_mixin<ramp> {
+  class ramp : public non_copyable_mixin {
    public:
     explicit ramp() = default;
     explicit ramp(vector<label_t>&& icons) : m_icons(forward<decltype(icons)>(icons)) {}
 
     void add(label_t&& icon);
+    void add(label_t&& icon, unsigned weight);
     label_t get(size_t index);
     label_t get_by_percentage(float percentage);
-    label_t get_by_percentage_with_borders(float percentage);
+    label_t get_by_percentage_with_borders(float percentage, float min, float max);
+    label_t get_by_percentage_with_borders(int percentage, int min, int max);
     operator bool();
 
    protected:
@@ -26,6 +28,6 @@ namespace drawtypes {
   using ramp_t = shared_ptr<ramp>;
 
   ramp_t load_ramp(const config& conf, const string& section, string name, bool required = true);
-}
+}  // namespace drawtypes
 
 POLYBAR_NS_END
