@@ -7,18 +7,20 @@ POLYBAR_NS
 
 namespace modules {
   enum class memtype { NONE = 0, TOTAL, USED, FREE, SHARED, BUFFERS, CACHE, AVAILABLE };
-
+  enum class memory_state { NORMAL = 0, WARN };
   class memory_module : public timer_module<memory_module> {
    public:
     explicit memory_module(const bar_settings&, string);
 
     bool update();
+    string get_format() const;
     bool build(builder* builder, const string& tag) const;
 
     static constexpr auto TYPE = "internal/memory";
 
    private:
     static constexpr const char* TAG_LABEL{"<label>"};
+    static constexpr const char* TAG_LABEL_WARN{"<label-warn>"};
     static constexpr const char* TAG_BAR_USED{"<bar-used>"};
     static constexpr const char* TAG_BAR_FREE{"<bar-free>"};
     static constexpr const char* TAG_RAMP_USED{"<ramp-used>"};
@@ -27,12 +29,15 @@ namespace modules {
     static constexpr const char* TAG_BAR_SWAP_FREE{"<bar-swap-free>"};
     static constexpr const char* TAG_RAMP_SWAP_USED{"<ramp-swap-used>"};
     static constexpr const char* TAG_RAMP_SWAP_FREE{"<ramp-swap-free>"};
+    static constexpr const char* FORMAT_WARN{"format-warn"};
 
     label_t m_label;
+    label_t m_labelwarn;
     progressbar_t m_bar_memused;
     progressbar_t m_bar_memfree;
     int m_perc_memused{0};
     int m_perc_memfree{0};
+    int m_perc_memused_warn{90};
     ramp_t m_ramp_memused;
     ramp_t m_ramp_memfree;
     progressbar_t m_bar_swapused;
