@@ -3,6 +3,7 @@
 #include "common.hpp"
 #include "components/config.hpp"
 #include "components/types.hpp"
+#include "drawtypes/layouticonset.hpp"
 #include "modules/meta/event_handler.hpp"
 #include "modules/meta/static_module.hpp"
 #include "x11/extensions/xkb.hpp"
@@ -34,11 +35,14 @@ namespace modules {
     bool query_keyboard();
     bool blacklisted(const string& indicator_name);
 
-    void handle(const evt::xkb_new_keyboard_notify& evt);
-    void handle(const evt::xkb_state_notify& evt);
-    void handle(const evt::xkb_indicator_state_notify& evt);
+    void handle(const evt::xkb_new_keyboard_notify& evt) override;
+    void handle(const evt::xkb_state_notify& evt) override;
+    void handle(const evt::xkb_indicator_state_notify& evt) override;
 
-    bool input(const string& action, const string& data);
+    void action_switch();
+
+    void define_layout_icon(const string& entry, const string& layout, const string& variant, label_t&& icon);
+    void parse_icons();
 
    private:
     static constexpr const char* TAG_LABEL_LAYOUT{"<label-layout>"};
@@ -61,7 +65,7 @@ namespace modules {
     map<keyboard::indicator::type, label_t> m_indicator_off_labels;
 
     vector<string> m_blacklist;
-    iconset_t m_layout_icons;
+    layouticonset_t m_layout_icons;
     iconset_t m_indicator_icons_on;
     iconset_t m_indicator_icons_off;
   };
