@@ -39,7 +39,7 @@ namespace modules {
     static constexpr auto TYPE = "internal/xwindow";
 
    protected:
-    void handle(const evt::property_notify& evt);
+    void handle(const evt::property_notify& evt) override;
 
    private:
     static constexpr const char* TAG_LABEL{"<label>"};
