@@ -2,10 +2,10 @@
 
 #include <poll.h>
 #include <sys/inotify.h>
+
 #include <cstdio>
 
 #include "common.hpp"
-#include "utils/factory.hpp"
 
 POLYBAR_NS
 
@@ -40,8 +40,8 @@ class inotify_watch {
 namespace inotify_util {
   template <typename... Args>
   decltype(auto) make_watch(Args&&... args) {
-    return factory_util::unique<inotify_watch>(forward<Args>(args)...);
+    return std::make_unique<inotify_watch>(forward<Args>(args)...);
   }
-}
+}  // namespace inotify_util
 
 POLYBAR_NS_END
