@@ -3,8 +3,6 @@
 #include "drawtypes/animation.hpp"
 #include "drawtypes/label.hpp"
 #include "drawtypes/ramp.hpp"
-#include "utils/factory.hpp"
-
 #include "modules/meta/base.inl"
 
 POLYBAR_NS
@@ -16,11 +14,50 @@ namespace modules {
       : timer_module<network_module>(bar, move(name_)) {
     // Load configuration values
     m_interface = m_conf.get(name(), "interface", m_interface);
+
+    if (m_interface.empty()) {
+      std::string type = m_conf.get(name(), "interface-type");
+      if (type == "wired") {
+        m_interface = net::find_wired_interface();
+        if (!m_interface.empty()) {
+          m_log.notice("%s: Discovered wired interface %s", name(), m_interface);
+        }
+      } else if (type == "wireless") {
+        m_interface = net::find_wireless_interface();
+        if (!m_interface.empty()) {
+          m_log.notice("%s: Discovered wireless interface %s", name(), m_interface);
+        }
+      } else {
+        throw module_error("Invalid interface type '" + type + "'");
+      }
+
+      if (m_interface.empty()) {
+        throw module_error("No interface found for type '" + type + "'");
+      }
+    }
+
+    if (m_interface.empty()) {
+      throw module_error("Missing 'interface' or 'interface-type'");
+    }
+
+    if (!net::is_interface_valid(m_interface)) {
+      throw module_error("Invalid network interface \"" + m_interface + "\"");
+    }
+
+    auto canonical = net::get_canonical_interface(m_interface);
+
+    if (canonical.second) {
+      m_log.info(
+          "%s: Replacing given interface '%s' with its canonical name '%s'", name(), m_interface, canonical.first);
+      m_interface = canonical.first;
+    }
+
     m_ping_nth_update = m_conf.get(name(), "ping-interval", m_ping_nth_update);
     m_udspeed_minwidth = m_conf.get(name(), "udspeed-minwidth", m_udspeed_minwidth);
     m_accumulate = m_conf.get(name(), "accumulate-stats", m_accumulate);
     set_interval(1s);
     m_unknown_up = m_conf.get<bool>(name(), "unknown-as-up", false);
+    m_udspeed_unit = m_conf.get<string>(name(), "speed-unit", m_udspeed_unit);
 
     m_conf.warn_deprecated(name(), "udspeed-minwidth", "%downspeed:min:max% and %upspeed:min:max%");
 
@@ -62,10 +99,10 @@ namespace modules {
 
     // Get an intstance of the network interface
     if (net::is_wireless_interface(m_interface)) {
-      m_wireless = factory_util::unique<net::wireless_network>(m_interface);
+      m_wireless = std::make_unique<net::wireless_network>(m_interface);
       m_wireless->set_unknown_up(m_unknown_up);
     } else {
-      m_wired = factory_util::unique<net::wired_network>(m_interface);
+      m_wired = std::make_unique<net::wired_network>(m_interface);
       m_wired->set_unknown_up(m_unknown_up);
     };
 
@@ -109,17 +146,20 @@ namespace modules {
       m_counter = 0;
     }
 
-    auto upspeed = network->upspeed(m_udspeed_minwidth);
-    auto downspeed = network->downspeed(m_udspeed_minwidth);
+    auto upspeed = network->upspeed(m_udspeed_minwidth, m_udspeed_unit);
+    auto downspeed = network->downspeed(m_udspeed_minwidth, m_udspeed_unit);
+    auto netspeed = network->netspeed(m_udspeed_minwidth, m_udspeed_unit);
 
     // Update label contents
     const auto replace_tokens = [&](label_t& label) {
       label->reset_tokens();
       label->replace_token("%ifname%", m_interface);
       label->replace_token("%local_ip%", network->ip());
+      label->replace_token("%mac%", network->mac());
       label->replace_token("%local_ip6%", network->ip6());
       label->replace_token("%upspeed%", upspeed);
       label->replace_token("%downspeed%", downspeed);
+      label->replace_token("%netspeed%", netspeed);
 
       if (m_wired) {
         label->replace_token("%linkspeed%", m_wired->linkspeed());
@@ -188,6 +228,6 @@ namespace modules {
 
     m_log.trace("%s: Reached end of network subthread", name());
   }
-}
+}  // namespace modules
 
 POLYBAR_NS_END
