@@ -1,10 +1,10 @@
+#include "utils/string.hpp"
+
 #include <algorithm>
 #include <iomanip>
 #include <sstream>
 #include <utility>
 
-#include "utils/string.hpp"
-
 POLYBAR_NS
 
 namespace string_util {
@@ -15,6 +15,21 @@ namespace string_util {
     return haystack.find(needle) != string::npos;
   }
 
+  bool ends_with(const string& haystack, const string& suffix) {
+    if (haystack.length() < suffix.length()) {
+      return false;
+    }
+
+    return haystack.compare(haystack.length() - suffix.length(), suffix.length(), suffix) == 0;
+  }
+
+  /**
+   * Check if haystack contains needle ignoring case
+   */
+  bool contains_ignore_case(const string& haystack, const string& needle) {
+    return lower(haystack).find(lower(needle)) != string::npos;
+  }
+
   /**
    * Convert string to uppercase
    */
@@ -45,7 +60,7 @@ namespace string_util {
   }
 
   /**
-   * Replace first occurence of needle in haystack
+   * Replace first occurrence of needle in haystack
    */
   string replace(const string& haystack, const string& needle, const string& replacement, size_t start, size_t end) {
     string str(haystack);
@@ -61,7 +76,7 @@ namespace string_util {
   }
 
   /**
-   * Replace all occurences of needle in haystack
+   * Replace all occurrences of needle in haystack
    */
   string replace_all(
       const string& haystack, const string& needle, const string& replacement, size_t start, size_t end) {
@@ -291,6 +306,18 @@ namespace string_util {
     return floating_point(kibibytes / 1024.0 / 1024.0, precision, true, locale) + " GiB";
   }
 
+  /**
+   * Create a GiB string, if the value in GiB is >= 1.0. Otherwise, create a MiB string.
+   */
+  string filesize_gib_mib(
+      unsigned long long kibibytes, size_t precision_mib, size_t precision_gib, const string& locale) {
+    if (kibibytes < 1024 * 1024) {
+      return filesize_mib(kibibytes, precision_mib, locale);
+    } else {
+      return filesize_gib(kibibytes, precision_gib, locale);
+    }
+  }
+
   /**
    * Create a filesize string by converting given bytes to highest unit possible
    */
