@@ -1,4 +1,5 @@
 #include "x11/icccm.hpp"
+
 #include "x11/atoms.hpp"
 
 POLYBAR_NS
@@ -33,11 +34,23 @@ namespace icccm_util {
   bool get_wm_urgency(xcb_connection_t* c, xcb_window_t w) {
     xcb_icccm_wm_hints_t hints;
     if (xcb_icccm_get_wm_hints_reply(c, xcb_icccm_get_wm_hints(c, w), &hints, NULL)) {
-      if(xcb_icccm_wm_hints_get_urgency(&hints) == XCB_ICCCM_WM_HINT_X_URGENCY)
+      if (xcb_icccm_wm_hints_get_urgency(&hints) == XCB_ICCCM_WM_HINT_X_URGENCY)
         return true;
     }
     return false;
   }
-}
+
+  void set_wm_size_hints(xcb_connection_t* c, xcb_window_t w, int x, int y, int width, int height) {
+    xcb_size_hints_t hints{};
+
+    xcb_icccm_size_hints_set_size(&hints, false, width, height);
+    xcb_icccm_size_hints_set_min_size(&hints, width, height);
+    xcb_icccm_size_hints_set_max_size(&hints, width, height);
+    xcb_icccm_size_hints_set_base_size(&hints, width, height);
+    xcb_icccm_size_hints_set_position(&hints, false, x, y);
+
+    xcb_icccm_set_wm_size_hints(c, w, XCB_ATOM_WM_NORMAL_HINTS, &hints);
+  }
+} // namespace icccm_util
 
 POLYBAR_NS_END
