@@ -1,8 +1,9 @@
 #include "utils/command.hpp"
-#include "common/test.hpp"
 
 #include <unistd.h>
 
+#include "common/test.hpp"
+
 using namespace polybar;
 
 TEST(Command, status) {
@@ -44,14 +45,3 @@ TEST(Command, output) {
 
   EXPECT_EQ(str, "polybar");
 }
-
-TEST(Command, readline) {
-  auto cmd = command_util::make_command<output_policy::REDIRECTED>("read text;echo $text");
-
-  string str;
-  cmd->exec(false);
-  cmd->writeline("polybar");
-  cmd->tail([&str](string&& string) { str = string; });
-
-  EXPECT_EQ(str, "polybar");
-}
