@@ -1,8 +1,18 @@
 #include "utils/string.hpp"
+
 #include "common/test.hpp"
 
 using namespace polybar;
 
+TEST(String, ends_with) {
+  EXPECT_TRUE(string_util::ends_with("foo", "foo"));
+  EXPECT_TRUE(string_util::ends_with("foobar", "bar"));
+  EXPECT_TRUE(string_util::ends_with("foobar", ""));
+  EXPECT_FALSE(string_util::ends_with("foo", "bar"));
+  EXPECT_FALSE(string_util::ends_with("foo", "Foo"));
+  EXPECT_FALSE(string_util::ends_with("", "Foo"));
+}
+
 TEST(String, upper) {
   EXPECT_EQ("FOO", string_util::upper("FOO"));
   EXPECT_EQ("FOO", string_util::upper("FoO"));
@@ -20,6 +30,32 @@ TEST(String, compare) {
   EXPECT_FALSE(string_util::compare("foo", "bar"));
 }
 
+TEST(String, contains) {
+  EXPECT_TRUE(string_util::contains("fooooobar", "foo"));
+  EXPECT_TRUE(string_util::contains("barrrrrrfoo", "foo"));
+  EXPECT_TRUE(string_util::contains("barrfoobazzz", "foo"));
+  EXPECT_TRUE(string_util::contains("foo", "foo"));
+  EXPECT_TRUE(string_util::contains("foobar", "foo"));
+  EXPECT_TRUE(string_util::contains("foobar", "bar"));
+  EXPECT_FALSE(string_util::contains("foo", "Foo"));
+  EXPECT_FALSE(string_util::contains("foo", "bar"));
+  EXPECT_FALSE(string_util::contains("foobar", "baz"));
+  EXPECT_FALSE(string_util::contains("foobAr", "bar"));
+}
+
+TEST(String, contains_ignore_case) {
+  EXPECT_TRUE(string_util::contains_ignore_case("fooooobar", "foo"));
+  EXPECT_TRUE(string_util::contains_ignore_case("barrrrrrfoo", "foo"));
+  EXPECT_TRUE(string_util::contains_ignore_case("barrfoobazzz", "foo"));
+  EXPECT_TRUE(string_util::contains_ignore_case("fooooobar", "fOO"));
+  EXPECT_TRUE(string_util::contains_ignore_case("barrrrrrfoo", "FOo"));
+  EXPECT_TRUE(string_util::contains_ignore_case("barrfoobazzz", "FoO"));
+  EXPECT_TRUE(string_util::contains_ignore_case("foo", "Foo"));
+  EXPECT_FALSE(string_util::contains_ignore_case("foo", "bar"));
+  EXPECT_TRUE(string_util::contains_ignore_case("foo", ""));
+  EXPECT_FALSE(string_util::contains_ignore_case("", "bar"));
+}
+
 TEST(String, replace) {
   EXPECT_EQ("a.c", string_util::replace("abc", "b", "."));
   EXPECT_EQ("a.a", string_util::replace("aaa", "a", ".", 1, 2));
